#ifndef MARSYASQT_COMMON_INCLUDED
#define MARSYASQT_COMMON_INCLUDED

#include <marsyas/types.h>

#include <QMetaType>

Q_DECLARE_METATYPE(Marsyas::mrs_real);
Q_DECLARE_METATYPE(Marsyas::mrs_natural);
Q_DECLARE_METATYPE(Marsyas::mrs_realvec);

#endif // MARSYASQT_COMMON_INCLUDED
